import { Inject, Type, Optional, Self, SkipSelf, InjectionToken, resolveForwardRef, Injector } from '@angular/core';
import { Mock, MoqAPI } from 'moq.ts';

class TokenNameProvider {
    getName(token) {
        if (typeof token === "string") {
            return token;
        }
        if (token instanceof Array) {
            return `[${token.map(this.getName).join(", ")}]`;
        }
        if (token == null) {
            return `${token}`;
        }
        if (token.overriddenName) {
            return `${token.overriddenName}`;
        }
        if (token.name) {
            return `${token.name}`;
        }
        const res = token.toString();
        if (res == null) {
            return `${res}`;
        }
        const newLineIndex = res.indexOf("\n");
        return newLineIndex === -1 ? res : res.substring(0, newLineIndex);
    }
}

/**
 * Provides dependencies metadata of inspected angular class.
 *
 * @param type The inspected class
 */
class Reflector {
    constructor(tokenName) {
        this.tokenName = tokenName;
        return this.factory();
    }
    static parameters(type) {
        const paramAnnotations = type.hasOwnProperty("__parameters__") && type["__parameters__"];
        const reflect = (global || window)["Reflect"];
        const paramTypes = reflect.getOwnMetadata("design:paramtypes", type);
        if (paramTypes || paramAnnotations) {
            return Reflector.zipTypesAndAnnotations(paramTypes, paramAnnotations);
        }
        return [];
    }
    static zipTypesAndAnnotations(paramTypes, paramAnnotations) {
        let result;
        if (typeof paramTypes === "undefined") {
            result = new Array(paramAnnotations.length);
        }
        else {
            result = new Array(paramTypes.length);
        }
        for (let i = 0; i < result.length; i++) {
            // TS outputs Object for parameters without types, while Traceur omits
            // the annotations. For now we preserve the Traceur behavior to aid
            // migration, but this can be revisited.
            if (typeof paramTypes === "undefined") {
                result[i] = [];
            }
            else if (paramTypes[i] !== Object) {
                result[i] = [paramTypes[i]];
            }
            else {
                result[i] = [];
            }
            if (paramAnnotations && paramAnnotations[i] != null) {
                result[i] = result[i].concat(paramAnnotations[i]);
            }
        }
        return result;
    }
    factory() {
        return (type) => {
            const params = Reflector.parameters(type);
            return params.map(p => this.extractToken(p));
        };
    }
    extractToken(metadata) {
        let token = null;
        let optional = false;
        if (!Array.isArray(metadata)) {
            if (metadata instanceof Inject) {
                return this.createDependency(metadata.token, optional, 0 /* None */);
            }
            else {
                return this.createDependency(metadata, optional, 0 /* None */);
            }
        }
        let visibility = 0 /* None */;
        for (const paramMetadata of metadata) {
            if (paramMetadata instanceof Type) {
                token = paramMetadata;
            }
            else if (paramMetadata instanceof Inject) {
                token = paramMetadata.token;
            }
            else if (paramMetadata instanceof Optional) {
                optional = true;
            }
            else if (paramMetadata instanceof Self) {
                visibility = 1 /* Self */;
            }
            else if (paramMetadata instanceof SkipSelf) {
                visibility = 2 /* SkipSelf */;
            }
            else if (paramMetadata instanceof InjectionToken) {
                token = paramMetadata;
            }
        }
        token = resolveForwardRef(token);
        return this.createDependency(token, optional, visibility);
    }
    createDependency(token, optional, visibility) {
        return { displayName: this.tokenName.getName(token), token, optional, visibility };
    }
}
Reflector.ctorParameters = () => [
    { type: TokenNameProvider, decorators: [{ type: Inject, args: [TokenNameProvider,] }] }
];

class TestedUnitProviderFactory {
    constructor() {
        return this.factory();
    }
    factory() {
        return (type, dependencies) => ({ provide: type, useClass: type, deps: [...dependencies] });
    }
}

class ProviderFactory {
    constructor() {
        return this.factory();
    }
    factory() {
        return (parameter, mocked) => ({ provide: parameter.token, useValue: mocked });
    }
}

class MockFactory {
    constructor() {
        return this.factory();
    }
    factory() {
        return (parameter) => new Mock({ name: parameter.displayName });
    }
}

class ParametersStaticProviderFactory {
    constructor(providerFactory, mockFactory) {
        this.providerFactory = providerFactory;
        this.mockFactory = mockFactory;
        return this.factory();
    }
    factory() {
        return (parameters) => {
            const providers = [];
            for (const parameter of parameters) {
                const mock = this.mockFactory(parameter);
                const staticProvider = this.providerFactory(parameter, mock.object());
                if (staticProvider !== undefined) {
                    providers.push(staticProvider);
                }
            }
            return providers;
        };
    }
}
ParametersStaticProviderFactory.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [ProviderFactory,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [MockFactory,] }] }
];

class DepsFactory {
    constructor() {
        return this.factory();
    }
    factory() {
        return (parameters) => parameters
            .map(p => {
            if (p.optional === true) {
                return [new Optional(), p.token];
            }
            return p.token;
        });
    }
}

class MoqInjectorProviders {
    constructor(reflector, testedUnitProviderFactory, depsFactory, parametersStaticProviderFactory) {
        this.reflector = reflector;
        this.testedUnitProviderFactory = testedUnitProviderFactory;
        this.depsFactory = depsFactory;
        this.parametersStaticProviderFactory = parametersStaticProviderFactory;
        return this.factory();
    }
    factory() {
        return (type, options = { skipSelf: false, skipOptional: false }) => {
            const parameters = this.reflector(type);
            const provider = this.testedUnitProviderFactory(type, this.depsFactory(parameters));
            const onlyRequiredParameters = options.skipOptional
                ? parameters.filter(({ optional }) => optional === false)
                : parameters;
            const providers = Array.from(this.parametersStaticProviderFactory(onlyRequiredParameters));
            return (options === null || options === void 0 ? void 0 : options.skipSelf) ? providers : [provider, ...providers];
        };
    }
}
MoqInjectorProviders.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [Reflector,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [TestedUnitProviderFactory,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [DepsFactory,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [ParametersStaticProviderFactory,] }] }
];

/**
 * Provides the service-to-test and its (moq) dependencies
 */
const moqInjectorProvidersFactory = (options = {}) => {
    const injector = Injector.create({
        providers: [
            {
                provide: MoqInjectorProviders,
                useClass: MoqInjectorProviders,
                deps: [Reflector, TestedUnitProviderFactory, DepsFactory, ParametersStaticProviderFactory]
            },
            { provide: TestedUnitProviderFactory, useClass: TestedUnitProviderFactory, deps: [] },
            { provide: DepsFactory, useClass: DepsFactory, deps: [] },
            { provide: ProviderFactory, useClass: ProviderFactory, deps: [] },
            { provide: ParametersStaticProviderFactory, useClass: ParametersStaticProviderFactory, deps: [ProviderFactory, MockFactory] },
            { provide: MockFactory, useClass: MockFactory, deps: [] },
            { provide: Reflector, useClass: Reflector, deps: [TokenNameProvider] },
            { provide: TokenNameProvider, useClass: TokenNameProvider, deps: [] },
            ...(options.providers || [])
        ]
    });
    return injector.get(MoqInjectorProviders);
};

/**
 * Gets an instance of IMock interface for mocked object according to provided token.
 * Injector should be built with {@link moqInjectorProviders}.
 *
 * @param token angular token
 * @param injector angular injector
 */
const resolveMock = (token, injector) => {
    const object = injector.get(token);
    return object[MoqAPI];
};

/*
 * Public API Surface of ng-auto-moq
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DepsFactory, MockFactory, MoqInjectorProviders, ParametersStaticProviderFactory, Reflector, TestedUnitProviderFactory, TokenNameProvider, moqInjectorProvidersFactory, resolveMock, ProviderFactory as ɵa };
//# sourceMappingURL=ng-auto-moq.js.map
